#ifndef QUAZIP_CONFIG_H
#define QUAZIP_CONFIG_H

/*
 * This file is generated by CMake. Do not edit manually.
 * It records build-time options that must be visible to header consumers.
 */

#define QUAZIP_CAN_USE_QTEXTCODEC

#endif // QUAZIP_CONFIG_H
